package de.twomartens.troubleshooting

import android.view.LayoutInflater
import android.view.ViewGroup
import androidx.recyclerview.widget.RecyclerView
import de.twomartens.troubleshooting.databinding.MessageReceivedBinding
import de.twomartens.troubleshooting.databinding.MessageSentBinding
import java.text.DateFormat

const val VIEW_TYPE_MESSAGE_RECEIVED: Int = 1
const val VIEW_TYPE_MESSAGE_SENT: Int = 2

class MessageListAdapter : RecyclerView.Adapter<RecyclerView.ViewHolder>() {
    private var messages: ArrayList<Message> = ArrayList()
    val dateFormat: DateFormat = DateFormat.getTimeInstance(DateFormat.SHORT)

    inner class ReceivedMessageHolder(private val binding: MessageReceivedBinding)
        : RecyclerView.ViewHolder(binding.root) {
        private var message: Message? = null

        fun bind(message: Message) {
            this.message = message
            binding.textMessageBody.text = message.message
            binding.textMessageTime.text = dateFormat.format(message.createdAt)
        }
    }

    inner class SentMessageHolder(private val binding: MessageSentBinding)
        : RecyclerView.ViewHolder(binding.root) {
        private var message: Message? = null

        fun bind(message: Message) {
            this.message = message
            binding.textMessageBody.text = message.message
            binding.textMessageTime.text = dateFormat.format(message.createdAt)
        }
    }


    // Create new views (invoked by the layout manager)
    override fun onCreateViewHolder(parent: ViewGroup,
                                    viewType: Int): RecyclerView.ViewHolder {
        return when (viewType) {
            VIEW_TYPE_MESSAGE_RECEIVED -> {
                val binding = MessageReceivedBinding.inflate(LayoutInflater.from(parent.context), parent, false)
                ReceivedMessageHolder(binding)
            }
            VIEW_TYPE_MESSAGE_SENT -> {
                val binding = MessageSentBinding.inflate(LayoutInflater.from(parent.context), parent, false)
                SentMessageHolder(binding)
            }
            else -> {
                val binding = MessageReceivedBinding.inflate(LayoutInflater.from(parent.context), parent, false)
                ReceivedMessageHolder(binding)
            }
        }
    }

    // Replace the contents of a view (invoked by the layout manager)
    override fun onBindViewHolder(holder: RecyclerView.ViewHolder, position: Int) {
        if (holder is SentMessageHolder) {
            holder.bind(messages[position])
        }
        if (holder is ReceivedMessageHolder) {
            holder.bind(messages[position])
        }
    }

    override fun getItemViewType(position: Int): Int {
        val message = messages[position]
        return if (message.sent) {
            VIEW_TYPE_MESSAGE_SENT
        } else {
            VIEW_TYPE_MESSAGE_RECEIVED
        }
    }

    // Return the size of your data set (invoked by the layout manager)
    override fun getItemCount() = messages.size

    fun replaceMessages(messages: ArrayList<Message>) {
        this.messages = messages
        notifyDataSetChanged()
    }
}
